import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
  Input,
} from '@angular/core';
import { MatPaginator } from '@angular/material/paginator';
import { MatTableDataSource } from '@angular/material/table';
import { fuseAnimations } from '@fuse/animations';
import { MemberBankWithdrawService } from 'app/modules/polyglot/member_bank_withdraw/services/member_bank_withdraw.service';
import { ListMemberBankWithdrawsRequest } from 'app/modules/proto/c2se/member_bank_withdraw_pb';
import {
  MemberBankWithdrawModel,
  MemberBankWithdrawState,
} from 'app/modules/proto/models/member_bank_withdraw_pb';
import { MemberModel } from 'app/modules/proto/models/member_pb';
import { Subject } from 'rxjs';

@Component({
  selector: 'withdraw-history',
  templateUrl: './withdraw-history.component.html',
  styles: [
    /* language=SCSS */
    `
      .withdraw-history-grid {
        grid-template-columns: 60px auto 40px;

        @screen sm {
          grid-template-columns: 100px auto 60px 60px;
        }

        @screen md {
          grid-template-columns: 100px 100px auto 60px 60px;
        }

        @screen lg {
          grid-template-columns: 100px 100px 100px 60px 60px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class WithdrawHistoryComponent
  implements OnInit, AfterViewInit, OnDestroy
{
  @Input()
  member!: MemberModel | undefined;

  @ViewChild(MatPaginator) private _paginator!: MatPaginator;

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  withdrawHistoryTableColumns: string[] = ['no', 'amount', 'createAt', 'state'];

  withdrawHistoryDataSource: MatTableDataSource<MemberBankWithdrawModel> =
    new MatTableDataSource();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _memberBankWithdrawService: MemberBankWithdrawService
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    const req = new ListMemberBankWithdrawsRequest();
    const search = new ListMemberBankWithdrawsRequest.Search();
    search.setMemberId(this.member!.getId());
    req.setSearch(search);
    this._memberBankWithdrawService
      .listMemberBankWithdraws(req)
      .then((result) => {
        const mbwdl = result.getMemberBankWithdrawsList();
        this.withdrawHistoryDataSource.data = mbwdl;

        this._changeDetectorRef.markForCheck();
      });
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {}

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }

  __getWithdrawState(withdraw: MemberBankWithdrawModel): string | undefined {
    const state = withdraw.getState();
    let result: string = '';

    switch (state) {
      case MemberBankWithdrawState.APPLICATION:
        result = '신청';
        break;
      case MemberBankWithdrawState.PENDING:
        result = '대기';
        break;
      case MemberBankWithdrawState.COMPLETE:
        result = '완료';
        break;
      default:
        break;
    }

    return result;
  }
}
